@@ -65,7 +65,7 @@ private slots:
     // Dock area permissions for DockWidgets and DockWidgetGroupWindows
     void dockPermissions();
 
-    // test floating tabs and item_tree consistency
+    // test floating tabs, item_tree and window title consistency
     void floatingTabs();
 
     // test hide & show
@@ -104,10 +104,12 @@ private:
     // move a dock widget
     void moveDockWidget(QDockWidget* dw, QPoint to, QPoint from = QPoint()) const;
 
+#ifdef QT_BUILD_INTERNAL
     // Message handling for xcb error QTBUG 82059
     static void xcbMessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg);
 public:
     bool xcbError = false;
+#endif
 private:
 
 #ifdef QT_DEBUG
@@ -128,11 +130,6 @@ private:
 
 };
 
-// Statics for xcb error / msg handler
-static tst_QDockWidget *qThis = nullptr;
-static void (*oldMessageHandler)(QtMsgType, const QMessageLogContext&, const QString&);
-#define QXCBVERIFY(cond) do { if (xcbError) QSKIP("Test skipped due to XCB error"); QVERIFY(cond); } while (0)
-
 // Testing get/set functions
 void tst_QDockWidget::getSetCheck()
 {
@@ -1307,20 +1304,6 @@ bool tst_QDockWidget::checkFloatingTabs(QMainWindow* mainWindow, QPointer<QDockW
     return true;
 }
 
-// detect xcb error
-// qt.qpa.xcb: internal error:  void QXcbWindow::setNetWmStateOnUnmappedWindow() called on mapped window
-void tst_QDockWidget::xcbMessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg)
-{
-    Q_ASSERT(oldMessageHandler);
-
-    if (type == QtWarningMsg && QString(context.category) == "qt.qpa.xcb" && msg.contains("internal error")) {
-        Q_ASSERT(qThis);
-        qThis->xcbError = true;
-    }
-
-    return oldMessageHandler(type, context, msg);
-}
-
 #endif // QT_BUILD_INTERNAL
 
 // test floating tabs and item_tree consistency
@@ -1362,8 +1345,6 @@ void tst_QDockWidget::floatingTabs()
 
     // Both dock widgets must no longer be floating
     // disabled due to flakiness on macOS and Windows
-    //QTRY_VERIFY(!d1->isFloating());
-    //QTRY_VERIFY(!d2->isFloating());
     if (d1->isFloating())
         qWarning("OS flakiness: D1 is docked and reports being floating");
     if (d2->isFloating())
@@ -1371,11 +1352,25 @@ void tst_QDockWidget::floatingTabs()
 
     // Now MainWindow has to have a floatingTab child
     QPointer<QDockWidgetGroupWindow> ftabs;
-    QTRY_VERIFY(checkFloatingTabs(mainWindow, ftabs, QList<QDockWidget*>() << d1 << d2));
+    QTRY_VERIFY(checkFloatingTabs(mainWindow, ftabs, QList<QDockWidget *>() << d1 << d2));
+
+    // Hide both dock widgets. Verify that the group window is also hidden.
+    qCDebug(lcTestDockWidget) << "*** Hide and show tabbed dock widgets ***";
+    d1->hide();
+    d2->hide();
+    QTRY_VERIFY(ftabs->isHidden());
+
+    // Show both dockwidgets again. Verify that the group window is visible.
+    d1->show();
+    d2->show();
+    QTRY_VERIFY(ftabs->isVisible());
 
     /*
      * replug both dock widgets into their initial position
-     * expected behavior: both docks are plugged and no longer floating
+     * expected behavior:
+       - both docks are plugged
+       - both docks are no longer floating
+       - title changes have been propagated
      */
 
 
@@ -1396,6 +1391,12 @@ void tst_QDockWidget::floatingTabs()
     QTRY_VERIFY(d1->isFloating());
     QTRY_VERIFY(!d2->isFloating());
 
+    // Change titles
+    static constexpr QLatin1StringView newD1("New D1");
+    static constexpr QLatin1StringView newD2("New D2");
+    d1->setWindowTitle(newD1);
+    d2->setWindowTitle(newD2);
+
     // Plug back into dock areas
     qCDebug(lcTestDockWidget) << "*** test plugging back to dock areas ***";
     qCDebug(lcTestDockWidget) << "Move d1 to left dock";
@@ -1415,22 +1416,49 @@ void tst_QDockWidget::floatingTabs()
     QTRY_VERIFY(!mainWindow->findChild<QDockWidgetGroupWindow*>());
     QTRY_VERIFY(ftabs.isNull());
 
+    // check window titles
+    QCOMPARE(d1->windowTitle(), newD1);
+    QCOMPARE(d2->windowTitle(), newD2);
+
     // Check if paths are consistent
     qCDebug(lcTestDockWidget) << "Checking path consistency" << layout->layoutState.indexOf(d1) << layout->layoutState.indexOf(d2);
 
     // Path1 must be identical
-    QTRY_VERIFY(path1 == layout->layoutState.indexOf(d1));
+    QTRY_COMPARE(path1, layout->layoutState.indexOf(d1));
 
     // d1 must have a gap item due to size change
-    QTRY_VERIFY(layout->layoutState.indexOf(d2) == QList<int>() << path2 << 0);
+    QTRY_COMPARE(layout->layoutState.indexOf(d2), QList<int>() << path2 << 0);
 #else
     QSKIP("test requires -developer-build option");
 #endif // QT_BUILD_INTERNAL
 }
 
+#ifdef QT_BUILD_INTERNAL
+// Statics for xcb error / msg handler
+static tst_QDockWidget *qThis = nullptr;
+static void (*oldMessageHandler)(QtMsgType, const QMessageLogContext&, const QString&);
+#define QXCBVERIFY(cond) do { if (xcbError) QSKIP("Test skipped due to XCB error"); QVERIFY(cond); } while (0)
+
+// detect xcb error
+// qt.qpa.xcb: internal error:  void QXcbWindow::setNetWmStateOnUnmappedWindow() called on mapped window
+void tst_QDockWidget::xcbMessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &msg)
+{
+    Q_ASSERT(oldMessageHandler);
+
+    if (type == QtWarningMsg && QString(context.category) == "qt.qpa.xcb" && msg.contains("internal error")) {
+        Q_ASSERT(qThis);
+        qThis->xcbError = true;
+    }
+
+    return oldMessageHandler(type, context, msg);
+}
+#endif // QT_BUILD_INTERNAL
+
 // test hide & show
 void tst_QDockWidget::hideAndShow()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Test skipped on Wayland.");
 #ifdef QT_BUILD_INTERNAL
     // Skip test if xcb error is launched
     qThis = this;
@@ -1494,6 +1522,8 @@ void tst_QDockWidget::hideAndShow()
 // test closing and deleting consistency
 void tst_QDockWidget::closeAndDelete()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Test skipped on Wayland.");
 #ifdef QT_BUILD_INTERNAL
     // Create a mainwindow with a central widget and two dock widgets
     QPointer<QDockWidget> d1;
@@ -1558,6 +1588,8 @@ void tst_QDockWidget::closeAndDelete()
 // Test dock area permissions
 void tst_QDockWidget::dockPermissions()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("Test skipped on Wayland.");
 #ifdef Q_OS_WIN
     QSKIP("Test skipped on Windows platforms");
 #endif // Q_OS_WIN
